# bbb imports
import zope.deprecation
zope.deprecation.moved('Products.Archetypes.Marshall', 'Plone 5.1')

from Products.Archetypes.Marshall import NonLoweringMessage
from Products.Archetypes.Marshall import formatRFC822Headers
from Products.Archetypes.Marshall import parseRFC822
from Products.Archetypes.Marshall import RFC822Marshaller
